#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "SDL_image.h"

#include "Blist.h"
#include "auxiliar.h"

#include "mog2.h"

FONTCONTAINER::FONTCONTAINER(char *filename,int s)
{
	char fontname[256];
	sprintf(fontname,"fonts/%s",filename);

	bitmap_font=0;
	ttf_font=0;
	name=0;

	if (fontname[strlen(fontname)-4]=='.' &&
		fontname[strlen(fontname)-3]=='t' &&
		fontname[strlen(fontname)-2]=='t' &&
		fontname[strlen(fontname)-1]=='f') {
		ttf_font=TTF_OpenFont(fontname,s);
		if (ttf_font!=0) {
			bitmap_font=0;
			name=new char[strlen(filename)+1];
			strcpy(name,filename);
			size=s;
		} /* if */ 
	} else {
		SDL_Surface *sfc=IMG_Load(fontname);
		if (sfc!=0) {
			bitmap_font=SDL_CreateRGBSurface(0,sfc->w,sfc->h,32,0,0,0,AMASK);
			SDL_SetAlpha(bitmap_font,0,SDL_ALPHA_OPAQUE);
			SDL_BlitSurface(sfc,0,bitmap_font,0);
			SDL_FreeSurface(sfc);
		} /* if */ 
		if (bitmap_font!=0) {
			// set alpha mask:
			int i,j,k,s;
			int mean;
			Uint32 color;
			Uint8 r,g,b,a;
			s=bitmap_font->w/256;

			for(k=0;k<128;k++) {
				for(i=0;i<s;i++) {
					for(j=0;j<sfc->h;j++) {
						SDL_LockSurface(bitmap_font);
						color=getpixel(bitmap_font,(k*2+1)*s+i,j);
						SDL_UnlockSurface(bitmap_font);
						SDL_GetRGBA(color,bitmap_font->format,&r,&g,&b,&a);
						mean=(r+g+b)/3;
						SDL_LockSurface(bitmap_font);
						color=getpixel(bitmap_font,(k*2)*s+i,j);
						SDL_UnlockSurface(bitmap_font);
						SDL_GetRGBA(color,bitmap_font->format,&r,&g,&b,&a);
						color=SDL_MapRGBA(bitmap_font->format,r,g,b,mean);
						SDL_LockSurface(bitmap_font);
						putpixel(bitmap_font,(k*2)*s+i,j,color);
						SDL_UnlockSurface(bitmap_font);
					} /* for */ 
				} /* for */ 
			} /* for */ 

			ttf_font=0;
			name=new char[strlen(filename)+1];
			strcpy(name,filename);
			size=bitmap_font->h;
		} /* if */ 
	} /* if */ 
} /* FONTCONTAINER::FONTCONTAINER */ 


FONTCONTAINER::~FONTCONTAINER(void)
{
	if (ttf_font!=0) TTF_CloseFont(ttf_font);
	if (bitmap_font!=0) SDL_FreeSurface(bitmap_font);
	delete []name;
	ttf_font=0;
	bitmap_font=0;
	name=0;
} /* FONTCONTAINER::~FONTCONTAINER */ 


void FONTCONTAINER::print_left(char *text,int r,int g,int b,SDL_Surface *sfc,int x,int y,int spacing)
{
	if (ttf_font!=0) {
		::print_left(text,ttf_font,r,g,b,sfc,x,y);
	} else {
		if (bitmap_font!=0) {
			print_left_bmp(text,bitmap_font,sfc,x,y,spacing);
		} /* if */ 
	} /* if */ 
} /* FONTCONTAINER::print_left */ 


void FONTCONTAINER::print_centered(char *text,int r,int g,int b,SDL_Surface *sfc,int x,int y,int spacing)
{
	if (ttf_font!=0) {
		::print_centered(text,ttf_font,r,g,b,sfc,x,y);
	} else {
		if (bitmap_font!=0) {
			print_centered_bmp(text,bitmap_font,sfc,x,y,spacing);
		} /* if */ 
	} /* if */ 
} /* FONTCONTAINER::print_centered */ 


int FONTCONTAINER::get_text_width(char *text,int spacing)
{
	if (ttf_font!=0) {
		return ::get_text_width(text,ttf_font);
	} else {
		if (bitmap_font!=0) {
			return get_text_width_bmp(text,bitmap_font,spacing);
		} else {
			return 0;
		} /* if */ 
	} /* if */ 

} /* FONTCONTAINER::get_text_width */ 



SDL_Surface *FONTCONTAINER::multiline_text_surface(char *text,int line_dist,SDL_Color c,int spacing)
{
	if (ttf_font!=0) {
		return ::multiline_text_surface(text,line_dist,ttf_font,c);
	} else {
		if (bitmap_font!=0) {
			return multiline_text_surface_bmp(text,line_dist,bitmap_font,c,spacing);
		} else {
			return 0;
		} /* if */ 
	} /* if */ 
} /* FONTCONTAINER::multiline_text_surface */ 


SDL_Surface *FONTCONTAINER::multiline_text_surface_limited(char *text,int line_dist,SDL_Color c,int spacing,int max_w)
{
	if (ttf_font!=0) {
		return ::multiline_text_surface_limited(text,line_dist,ttf_font,c,max_w);
	} else {
		if (bitmap_font!=0) {
			return multiline_text_surface_limited_bmp(text,line_dist,bitmap_font,c,spacing,max_w);;
		} else {
			return 0;
		} /* if */ 
	} /* if */ 
} /* FONTCONTAINER::multiline_text_surface_limited */ 

